@@ -1,10 +1,9 @@
 /*
  * collectd - src/apcups.c
- * Copyright (C) 2007 Florian octo Forster
- * Copyright (C) 2006 Anthony Gialluca <tonyabg at charter.net>
- * Copyright (C) 2000-2004 Kern Sibbald
- * Copyright (C) 1996-99 Andre M. Hedrick <andre at suse.com>
- *
+ * Copyright (C) 2006-2007  Florian octo Forster
+ * Copyright (C) 2006       Anthony Gialluca <tonyabg at charter.net>
+ * Copyright (C) 2000-2004  Kern Sibbald
+ * Copyright (C) 1996-1999  Andre M. Hedrick <andre at suse.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General
@@ -22,15 +21,9 @@
  *
  * Authors:
  *   Anthony Gialluca <tonyabg at charter.net>
+ *   Florian octo Forster <octo at verplant.org>
  **/
 
-/*
- * FIXME: Don't know why but without this here atof() was not returning
- * correct values for me. This is behavior that I don't understand and
- * should be examined in closer detail.
- */
-#include <stdlib.h>
-
 #include "collectd.h"
 #include "common.h"      /* rrd_update_file */
 #include "plugin.h"      /* plugin_register, plugin_submit */
@@ -124,8 +117,7 @@ static int net_open (char *host, int port)
 	assert ((port > 0x00000000) && (port <= 0x0000FFFF));
 
 	/* Convert the port to a string */
-	snprintf (port_str, 8, "%i", port);
-	port_str[7] = '\0';
+	ssnprintf (port_str, sizeof (port_str), "%i", port);
 
 	/* Resolve name */
 	memset ((void *) &ai_hints, '\0', sizeof (ai_hints));
@@ -372,13 +364,13 @@ static void apc_submit_generic (char *type, char *type_inst, double value)
 
 	vl.values = values;
 	vl.values_len = 1;
-	vl.time = time (NULL);
 	sstrncpy (vl.host, hostname_g, sizeof (vl.host));
 	sstrncpy (vl.plugin, "apcups", sizeof (vl.plugin));
 	sstrncpy (vl.plugin_instance, "", sizeof (vl.plugin_instance));
-	strncpy (vl.type_instance, type_inst, sizeof (vl.type_instance));
+	sstrncpy (vl.type, type, sizeof (vl.type));
+	sstrncpy (vl.type_instance, type_inst, sizeof (vl.type_instance));
 
-	plugin_dispatch_values (type, &vl);
+	plugin_dispatch_values (&vl);
 }
 
 static void apc_submit (struct apc_detail_s *apcups_detail)
