@@ -27,7 +27,9 @@
 /*
  * Includes (for data types)
  */
-#include <stdint.h>
+#if HAVE_STDINT_H
+# include <stdint.h>
+#endif
 #include <inttypes.h>
 #include <time.h>
 
