@@ -342,7 +342,8 @@ static int mysql_config (oconfig_item_t *ci) /* {{{ */
 		else
 			sstrncpy (cb_name, "mysql", sizeof (cb_name));
 
-		plugin_register_complex_read (cb_name, mysql_read,
+		plugin_register_complex_read (/* group = */ NULL, cb_name,
+					      mysql_read,
 					      /* interval = */ NULL, &ud);
 	}
 	else
@@ -397,7 +398,7 @@ static MYSQL *getconnection (mysql_database_t *db)
 	}
 	else
 	{
-		INFO ("mysql plugin: Sucessfully connected to database %s "
+		INFO ("mysql plugin: Successfully connected to database %s "
 				"at server %s (server version: %s, protocol version: %d)",
 				(db->database != NULL) ? db->database : "<none>",
 				mysql_get_host_info (db->con),
