@@ -224,9 +224,7 @@ static int protocols_config (const char *key, const char *value)
   else if (strcasecmp (key, "IgnoreSelected") == 0)
   {
     int invert = 1;
-    if ((strcasecmp (value, "True") == 0)
-        || (strcasecmp (value, "Yes") == 0)
-        || (strcasecmp (value, "On") == 0))
+    if (IS_TRUE (value))
       invert = 0;
     ignorelist_set_invert (values_list, invert);
   }
