@@ -96,7 +96,7 @@ int kvm_pagesize;
 #elif HAVE_PERFSTAT
 static int pagesize;
 static perfstat_memory_total_t pmemory;
-/*# endif HAVE_PERFSTAT */ 
+/*# endif HAVE_PERFSTAT */
 
 #else
 # error "No applicable input method."
@@ -193,6 +193,8 @@ static int swap_read (void)
 	char *fields[8];
 	int numfields;
 
+	_Bool old_kernel=0;
+
 	derive_t swap_used   = 0;
 	derive_t swap_cached = 0;
 	derive_t swap_free   = 0;
@@ -208,25 +210,18 @@ static int swap_read (void)
 		return (-1);
 	}
 
-	while (fgets (buffer, 1024, fh) != NULL)
+	while (fgets (buffer, sizeof (buffer), fh) != NULL)
 	{
-		derive_t *val = NULL;
-
-		if (strncasecmp (buffer, "SwapTotal:", 10) == 0)
-			val = &swap_total;
-		else if (strncasecmp (buffer, "SwapFree:", 9) == 0)
-			val = &swap_free;
-		else if (strncasecmp (buffer, "SwapCached:", 11) == 0)
-			val = &swap_cached;
-		else
-			continue;
-
-		numfields = strsplit (buffer, fields, 8);
-
+		numfields = strsplit (buffer, fields, STATIC_ARRAY_SIZE (fields));
 		if (numfields < 2)
 			continue;
 
-		*val = (derive_t) atoll (fields[1]) * 1024LL;
+		if (strcasecmp (fields[0], "SwapTotal:") == 0)
+			strtoderive (fields[1], &swap_total);
+		else if (strcasecmp (fields[0], "SwapFree:") == 0)
+			strtoderive (fields[1], &swap_free);
+		else if (strcasecmp (fields[0], "SwapCached:") == 0)
+			strtoderive (fields[1], &swap_cached);
 	}
 
 	if (fclose (fh))
@@ -243,31 +238,45 @@ static int swap_read (void)
 
 	if ((fh = fopen ("/proc/vmstat", "r")) == NULL)
 	{
-		char errbuf[1024];
-		WARNING ("swap: fopen: %s",
-				sstrerror (errno, errbuf, sizeof (errbuf)));
-		return (-1);
-	}
-
-	while (fgets (buffer, 1024, fh) != NULL)
-	{
-		derive_t *val = NULL;
-
-		if (strncasecmp (buffer, "pswpin", 6) == 0)
-			val = &swap_in;
-		else if (strncasecmp (buffer, "pswpout", 7) == 0)
-			val = &swap_out;
+		// /proc/vmstat does not exist in kernels <2.6
+		if ((fh = fopen ("/proc/stat", "r")) == NULL )
+		{
+			char errbuf[1024];
+			WARNING ("swap: fopen: %s",
+					sstrerror (errno, errbuf, sizeof (errbuf)));
+			return (-1);
+		}
 		else
-			continue;
-
-		numfields = strsplit (buffer, fields, 8);
-
-		if (numfields < 2)
-			continue;
-
-		*val = (derive_t) atoll (fields[1]);
+			old_kernel = 1;
 	}
 
+	while (fgets (buffer, sizeof (buffer), fh) != NULL)
+	{
+		numfields = strsplit (buffer, fields, STATIC_ARRAY_SIZE (fields));
+
+		if (!old_kernel)
+		{
+			if (numfields != 2)
+				continue;
+
+			if (strcasecmp ("pswpin", fields[0]) != 0)
+				strtoderive (fields[1], &swap_in);
+			else if (strcasecmp ("pswpout", fields[0]) == 0)
+				strtoderive (fields[1], &swap_out);
+		}
+		else /* if (old_kernel) */
+		{
+			if (numfields != 3)
+				continue;
+
+			if (strcasecmp ("page", fields[0]) == 0)
+			{
+				strtoderive (fields[1], &swap_in);
+				strtoderive (fields[2], &swap_out);
+			}
+		}
+	} /* while (fgets) */
+
 	if (fclose (fh))
 	{
 		char errbuf[1024];
@@ -275,12 +284,11 @@ static int swap_read (void)
 				sstrerror (errno, errbuf, sizeof (errbuf)));
 	}
 
-	swap_submit ("used", swap_used, DS_TYPE_GAUGE);
-	swap_submit ("free", swap_free, DS_TYPE_GAUGE);
-	swap_submit ("cached", swap_cached, DS_TYPE_GAUGE);
-	swap_submit ("in", swap_in, DS_TYPE_DERIVE);
+	swap_submit ("used",   1024 * swap_used,   DS_TYPE_GAUGE);
+	swap_submit ("free",   1024 * swap_free,   DS_TYPE_GAUGE);
+	swap_submit ("cached", 1024 * swap_cached, DS_TYPE_GAUGE);
+	swap_submit ("in",  swap_in,  DS_TYPE_DERIVE);
 	swap_submit ("out", swap_out, DS_TYPE_DERIVE);
-
 /* #endif KERNEL_LINUX */
 
 #elif HAVE_LIBKSTAT
@@ -310,7 +318,7 @@ static int swap_read (void)
 	 * However, Solaris does not allow to allocated/reserved more than the
 	 * available swap (physical memory + disk swap), so the pedant may
 	 * prefer: allocated + unallocated = reserved, available
-	 * 
+	 *
 	 * We map the above to: used + resv = n/a, free
 	 *
 	 * Does your brain hurt yet?  - Christophe Kalt
