@@ -129,19 +129,20 @@ static void tss2_submit_gauge (const char *plugin_instance,
 
 	vl.values     = values;
 	vl.values_len = 1;
-	vl.time       = time (NULL);
 	sstrncpy (vl.host, hostname_g, sizeof (vl.host));
 	sstrncpy (vl.plugin, "teamspeak2", sizeof (vl.plugin));
 
 	if (plugin_instance != NULL)
 		sstrncpy (vl.plugin_instance, plugin_instance,
 				sizeof (vl.plugin_instance));
-	
+
+	sstrncpy (vl.type, type, sizeof (vl.type));
+
 	if (type_instance != NULL)
 		sstrncpy (vl.type_instance, type_instance,
 				sizeof (vl.type_instance));
 	
-	plugin_dispatch_values (type, &vl);
+	plugin_dispatch_values (&vl);
 } /* void tss2_submit_gauge */
 
 static void tss2_submit_io (const char *plugin_instance, const char *type,
@@ -158,15 +159,16 @@ static void tss2_submit_io (const char *plugin_instance, const char *type,
 
 	vl.values     = values;
 	vl.values_len = 2;
-	vl.time       = time (NULL);
 	sstrncpy (vl.host, hostname_g, sizeof (vl.host));
 	sstrncpy (vl.plugin, "teamspeak2", sizeof (vl.plugin));
 
 	if (plugin_instance != NULL)
 		sstrncpy (vl.plugin_instance, plugin_instance,
 				sizeof (vl.plugin_instance));
-	
-	plugin_dispatch_values (type, &vl);
+
+	sstrncpy (vl.type, type, sizeof (vl.type));
+
+	plugin_dispatch_values (&vl);
 } /* void tss2_submit_gauge */
 
 static void tss2_close_socket (void)
@@ -372,8 +374,7 @@ static int tss2_select_vserver (FILE *read_fh, FILE *write_fh, vserver_list_t *v
 	int status;
 
 	/* Send request */
-	snprintf (command, sizeof (command), "sel %i\r\n", vserver->port);
-	command[sizeof (command) - 1] = 0;
+	ssnprintf (command, sizeof (command), "sel %i\r\n", vserver->port);
 
 	status = tss2_send_request (write_fh, command);
 	if (status != 0)
@@ -405,7 +406,7 @@ static int tss2_select_vserver (FILE *read_fh, FILE *write_fh, vserver_list_t *v
 } /* int tss2_select_vserver */
 
 static int tss2_vserver_gapl (FILE *read_fh, FILE *write_fh,
-		vserver_list_t *vserver, gauge_t *ret_value)
+		gauge_t *ret_value)
 {
 	/*
 	 * Reads the vserver's average packet loss and submits it to collectd.
@@ -527,9 +528,8 @@ static int tss2_read_vserver (vserver_list_t *vserver)
 	else
 	{
 		/* Request server information */
-		snprintf (plugin_instance, sizeof (plugin_instance), "vserver%i",
+		ssnprintf (plugin_instance, sizeof (plugin_instance), "vserver%i",
 				vserver->port);
-		plugin_instance[sizeof (plugin_instance) - 1] = 0;
 
 		/* Select the server */
 		status = tss2_select_vserver (read_fh, write_fh, vserver);
@@ -677,7 +677,7 @@ static int tss2_read_vserver (vserver_list_t *vserver)
 	 * with an error. */
 	if ((status == 0) && (vserver != NULL))
 	{
-		status = tss2_vserver_gapl (read_fh, write_fh, vserver, &packet_loss);
+		status = tss2_vserver_gapl (read_fh, write_fh, &packet_loss);
 		if (status == 0)
 		{
 			valid |= 0x20;
