@@ -22,7 +22,9 @@
 #ifndef UTILS_CMD_FLUSH_H
 #define UTILS_CMD_FLUSH_H 1
 
-int handle_flush (FILE *fh, char **fields, int fields_num);
+#include <stdio.h>
+
+int handle_flush (FILE *fh, char *buffer);
 
 #endif /* UTILS_CMD_FLUSH_H */
 
