@@ -1,6 +1,7 @@
 /**
  * collectd - src/vserver.c
  * Copyright (C) 2006,2007  Sebastian Harl
+ * Copyright (C) 2007,2008  Florian octo Forster
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -17,6 +18,7 @@
  *
  * Authors:
  *   Sebastian Harl <sh at tokkee.org>
+ *   Florian octo Forster <octo at verplant.org>
  **/
 
 #include "collectd.h"
@@ -56,13 +58,13 @@ static void traffic_submit (const char *plugin_instance,
 
 	vl.values = values;
 	vl.values_len = STATIC_ARRAY_SIZE (values);
-	vl.time = time (NULL);
 	sstrncpy (vl.host, hostname_g, sizeof (vl.host));
 	sstrncpy (vl.plugin, "vserver", sizeof (vl.plugin));
-	strncpy (vl.plugin_instance, plugin_instance, sizeof (vl.plugin_instance));
-	strncpy (vl.type_instance, type_instance, sizeof (vl.type_instance));
+	sstrncpy (vl.plugin_instance, plugin_instance, sizeof (vl.plugin_instance));
+	sstrncpy (vl.type, "if_octets", sizeof (vl.type));
+	sstrncpy (vl.type_instance, type_instance, sizeof (vl.type_instance));
 
-	plugin_dispatch_values ("if_octets", &vl);
+	plugin_dispatch_values (&vl);
 } /* void traffic_submit */
 
 static void load_submit (const char *plugin_instance,
@@ -77,12 +79,12 @@ static void load_submit (const char *plugin_instance,
 
 	vl.values = values;
 	vl.values_len = STATIC_ARRAY_SIZE (values);
-	vl.time = time (NULL);
 	sstrncpy (vl.host, hostname_g, sizeof (vl.host));
 	sstrncpy (vl.plugin, "vserver", sizeof (vl.plugin));
-	strncpy (vl.plugin_instance, plugin_instance, sizeof (vl.plugin_instance));
+	sstrncpy (vl.plugin_instance, plugin_instance, sizeof (vl.plugin_instance));
+	sstrncpy (vl.type, "load", sizeof (vl.type));
 
-	plugin_dispatch_values ("load", &vl);
+	plugin_dispatch_values (&vl);
 }
 
 static void submit_gauge (const char *plugin_instance, const char *type,
@@ -96,13 +98,13 @@ static void submit_gauge (const char *plugin_instance, const char *type,
 
 	vl.values = values;
 	vl.values_len = STATIC_ARRAY_SIZE (values);
-	vl.time = time (NULL);
 	sstrncpy (vl.host, hostname_g, sizeof (vl.host));
 	sstrncpy (vl.plugin, "vserver", sizeof (vl.plugin));
-	strncpy (vl.plugin_instance, plugin_instance, sizeof (vl.plugin_instance));
-	strncpy (vl.type_instance, type_instance, sizeof (vl.type_instance));
+	sstrncpy (vl.plugin_instance, plugin_instance, sizeof (vl.plugin_instance));
+	sstrncpy (vl.type, type, sizeof (vl.type));
+	sstrncpy (vl.type_instance, type_instance, sizeof (vl.type_instance));
 
-	plugin_dispatch_values (type, &vl);
+	plugin_dispatch_values (&vl);
 } /* void submit_gauge */
 
 static inline long long __get_sock_bytes(const char *s)
@@ -139,7 +141,7 @@ static int vserver_read (void)
 
 	while (42)
 	{
-		size_t len;
+		int len;
 		char file[BUFSIZE];
 
 		FILE *fh;
@@ -168,7 +170,7 @@ static int vserver_read (void)
 		if (dent->d_name[0] == '.')
 			continue;
 
-		len = snprintf (file, sizeof (file), PROCDIR "/%s", dent->d_name);
+		len = ssnprintf (file, sizeof (file), PROCDIR "/%s", dent->d_name);
 		if ((len < 0) || (len >= BUFSIZE))
 			continue;
 		
@@ -185,8 +187,9 @@ static int vserver_read (void)
 			continue;
 
 		/* socket message accounting */
-		len = snprintf (file, BUFSIZE, PROCDIR "/%s/cacct", dent->d_name);
-		if ((len < 0) || (len >= BUFSIZE))
+		len = ssnprintf (file, sizeof (file),
+				PROCDIR "/%s/cacct", dent->d_name);
+		if ((len < 0) || ((size_t) len >= sizeof (file)))
 			continue;
 
 		if (NULL == (fh = fopen (file, "r")))
@@ -232,8 +235,9 @@ static int vserver_read (void)
 		}
 
 		/* thread information and load */
-		len = snprintf (file, BUFSIZE, PROCDIR "/%s/cvirt", dent->d_name);
-		if ((len < 0) || (len >= BUFSIZE))
+		len = ssnprintf (file, sizeof (file),
+				PROCDIR "/%s/cvirt", dent->d_name);
+		if ((len < 0) || ((size_t) len >= sizeof (file)))
 			continue;
 
 		if (NULL == (fh = fopen (file, "r")))
@@ -284,8 +288,9 @@ static int vserver_read (void)
 		}
 
 		/* processes and memory usage */
-		len = snprintf (file, BUFSIZE, PROCDIR "/%s/limit", dent->d_name);
-		if ((len < 0) || (len >= BUFSIZE))
+		len = ssnprintf (file, sizeof (file),
+				PROCDIR "/%s/limit", dent->d_name);
+		if ((len < 0) || ((size_t) len >= sizeof (file)))
 			continue;
 
 		if (NULL == (fh = fopen (file, "r")))
